from unittest.mock import PropertyMock, patch

from django.test import TestCase

from app_utils.testing import create_user_from_evecharacter

from squads.models import Groups, Memberships, Pending, filters
from squads.tests.testdata.load_allianceauth import load_allianceauth
from squads.tests.testdata.load_eveuniverse import load_eveuniverse
from squads.tests.testdata.load_filters import load_filters
from squads.tests.testdata.load_groups import load_groups
from squads.tests.testdata.load_memberaudit import load_memberaudit
from squads.tests.testdata.load_users import load_users


class GroupsModelTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_memberaudit()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )

    def test_group_creation(self):
        group = Groups.objects.create(
            name="Test Group",
            owner=self.user,
            description="A test group",
        )
        self.assertEqual(group.name, "Test Group")
        self.assertEqual(group.owner, self.user)
        self.assertEqual(group.description, "A test group")
        self.assertTrue(group.is_active)
        self.assertFalse(group.req_approve)
        self.assertIsNotNone(group.created_at)
        self.assertIsNotNone(group.updated_at)

    def test_str_method(self):
        group = Groups.objects.create(name="Test Group", owner=self.user)
        self.assertEqual(str(group), "Test Group")


class MemberShipModelTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_memberaudit()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )

    def test_membership_creation(self):
        group = Groups.objects.create(name="Test Group", owner=self.user)
        membership = Memberships.objects.create(group=group, user=self.user)
        self.assertEqual(membership.group, group)
        self.assertEqual(membership.user, self.user)
        self.assertIsNotNone(membership.joined_at)

    def test_str_method(self):
        group = Groups.objects.create(name="Test Group", owner=self.user)
        membership = Memberships.objects.create(group=group, user=self.user)
        self.assertEqual(str(membership), f"{membership.user} in {membership.group}")


class PendingModelTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_memberaudit()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )

    def test_pending_creation(self):
        group = Groups.objects.create(name="Test Group", owner=self.user)
        pending = Pending.objects.create(group=group, user=self.user)
        self.assertEqual(pending.group, group)
        self.assertEqual(pending.user, self.user)
        self.assertIsNotNone(pending.created_at)

    def test_str_method(self):
        group = Groups.objects.create(name="Test Group", owner=self.user)
        pending = Pending.objects.create(group=group, user=self.user)
        self.assertEqual(str(pending), f"{pending.user} pending {pending.group}")


class FilterModelTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()
        load_memberaudit()
        load_users()
        try:
            load_groups()
            load_filters()
        except Exception as e:
            print(e)

        cls.user, _ = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )
        cls.user2, _ = create_user_from_evecharacter(
            1004,
            permissions=[
                "squads.basic_access",
            ],
        )
        cls.user3, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "squads.basic_access",
            ],
        )

    def test_squad_group_str_method(self):
        # given/when
        filter = filters.SquadGroup.objects.first()
        # then
        self.assertEqual(str(filter), f"Filter Group: {filter.group.name}")

    def test_squad_filter_str_method(self):
        # given/when
        filter = filters.SquadFilter.objects.first()
        # then
        self.assertEqual(str(filter), f"{filter.filter_object}")

    def test_skill_check_filter_valid(self):
        # given
        filter = filters.SkillSetFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user)
        # then
        self.assertTrue(filter_req)

    def test_skill_check_filter_not_valid_but_skilled(self):
        # given
        filter = filters.SkillSetFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user3)
        # then
        self.assertFalse(filter_req)

    def test_skill_check_filter_not_valid(self):
        # given
        filter = filters.SkillSetFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user2)
        # then
        self.assertFalse(filter_req)

    def test_asset_check_filter_valid(self):
        # given
        filter = filters.AssetsFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user)
        # then
        self.assertTrue(filter_req)

    def test_asset_check_filter_not_valid(self):
        # given
        filter = filters.AssetsFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user2)
        # then
        self.assertFalse(filter_req)

    def test_ship_check_filter_valid(self):
        # given
        filter = filters.ShipFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user)
        # then
        self.assertTrue(filter_req)

    def test_ship_check_filter_not_valid(self):
        # given
        filter = filters.ShipFilter.objects.first()
        # when
        filter_req, _ = filter.check_filter(self.user2)
        # then
        self.assertFalse(filter_req)

    def test_squad_run_filters(self):
        # given
        group = filters.SquadGroup.objects.first()
        # when
        filter_req, _ = group.check_user(self.user)
        # then
        self.assertTrue(filter_req)

    def test_squad_run_filters_not_valid(self):
        # given
        group = filters.SquadGroup.objects.first()
        # when
        filter_req, _ = group.check_user(self.user2)
        # then
        self.assertFalse(filter_req)

    @patch("squads.models.filters.SquadFilter.filter_object")
    def test_squad_run_filters_error_filter(self, mock_filter_object):
        # given
        group = filters.SquadGroup.objects.first()
        mock_filter_object.check_filter.side_effect = Exception("Error")
        # when
        filter_req, _ = group.check_user(self.user2)
        # then
        self.assertFalse(filter_req)



"""Manage views."""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import get_object_or_404, redirect

from squads.forms import CommentForm
from squads.hooks import get_extension_logger
from squads.models.groups import Groups
from squads.models.member import Memberships, Pending

logger = get_extension_logger(__name__)


@login_required
@permission_required("squads.basic_access")
def apply_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    comment_form = CommentForm(request.POST or None)
    if request.method == "POST" and "apply_group" in request.POST:
        user = request.user
        comment = ""
        if comment_form.is_valid():
            comment = comment_form.cleaned_data.get("comment")
        Pending.objects.create(group=group, user=user, comment=comment)
        messages.success(request, "Your application has been submitted.")
        return redirect("squads:view_group", group_id=group_id)

    if request.method == "POST" and "join_group" in request.POST:
        Memberships.objects.create(
            group=group,
            user=request.user,
            req_filters=True,
            is_active=True,
        )
        messages.success(request, f"You joined {group.name}.")
        return redirect("squads:view_group", group_id=group_id)
    return redirect("squads:view_group", group_id=group_id)


@login_required
@permission_required("squads.basic_access")
def leave_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    membership = Memberships.objects.filter(user=request.user, group=group).first()

    if membership:
        membership.delete()
        messages.warning(request, f"You Left {membership.group.name}.")
    else:
        messages.error(request, "You are not in that Squad.")

    return redirect("squads:view_group", group_id=group_id)


@login_required
@permission_required("squads.basic_access")
def cancel_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    pending = Pending.objects.filter(user=request.user, group=group).first()

    if pending:
        pending.delete()
        messages.warning(
            request, f"You canceled your application to {pending.group.name}."
        )
    else:
        messages.error(request, "You are not applied to that Squad.")

    return redirect("squads:view_group", group_id=group_id)



from http import HTTPStatus
from unittest.mock import MagicMock, patch

from django.contrib.auth.models import User
from django.contrib.staticfiles import finders
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from squads.models import Groups
from squads.tests.testdata.load_groups import load_groups
from squads.tests.testdata.load_users import load_users
from squads.views.groups import broswe_groups, create_group, view_group


class GroupViewTests(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        load_groups()
        cls.factory = RequestFactory()
        cls.user = User.objects.get(username="groupuser")
        cls.user2 = User.objects.get(username="groupuser2")
        cls.group = Groups.objects.get(id=1)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", cls.user)

    @patch("squads.views.groups.messages")
    def test_create_group(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:create_group"),
            {
                "name": "New Test Group",
                "description": "Test Description",
                "req_approve": True,
            },
        )
        request.user = self.user
        # when
        response = create_group(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertTrue(
            Groups.objects.filter(
                name="New Test Group",
                description="Test Description",
                req_approve=True,
            ).exists()
        )
        mock_messages.success.assert_called_once()

    def test_create_group_invalid_form(self):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:create_group"),
            {
                "name": "",
                "description": "Test Description",
                "req_approve": True,
            },
        )
        request.user = self.user
        # when
        response = create_group(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertFalse(
            Groups.objects.filter(
                name="New Test Group",
                description="Test Description",
                req_approve=True,
            ).exists()
        )

    def test_create_group_no_post(self):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:create_group"))
        request.user = self.user
        # when
        response = create_group(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertFalse(
            Groups.objects.filter(
                name="New Test Group",
                description="Test Description",
                req_approve=True,
            ).exists()
        )

    @patch("squads.views.groups.messages")
    @patch("squads.views.groups.SquadsGroupForm")
    def test_create_group_with_image(self, mock_group, mock_messages):
        # given
        from django.core.files.uploadedfile import SimpleUploadedFile

        self.client.force_login(self.user)
        image_path = finders.find("squads/groups_images/empty.png")
        with open(image_path, "rb") as img:
            uploaded_image = SimpleUploadedFile(
                name="empty.png", content=img.read(), content_type="image/png"
            )
        mock_group = MagicMock()
        mock_group.is_valid.return_value = True
        mock_group.save.return_value = Groups(
            name="New Test Group2",
            description="Test Description",
            req_approve=True,
            image=uploaded_image,
        )
        data = {
            "name": "New Test Group2",
            "description": "Test Description",
            "req_approve": True,
            "image": uploaded_image,
        }
        request = self.factory.post(reverse("squads:create_group"), data)
        request.user = self.user
        request.FILES["image"] = uploaded_image
        # when
        response = create_group(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.success.assert_called_once()

    def test_browse_groups(self):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:groups"))
        request.user = self.user
        # when
        response = broswe_groups(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group Approve")
        self.assertNotContains(response, "Group No Approve")
        self.assertContains(response, "Group Superuser")
        self.assertNotContains(response, "Group Staff")

    def test_browse_groups_many_pages(self):
        # given
        for i in range(70):
            Groups.objects.create(owner=self.user, name=f"Group {i}", is_active=True)
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:groups"), {"page": "1"})
        request.user = self.user
        # when
        response = broswe_groups(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group 50")
        # given 2
        request = self.factory.get(reverse("squads:groups"), {"page": "2"})
        request.user = self.user
        # when 2
        response = broswe_groups(request)
        # then 2
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group 60")

    def test_browse_groups_empty_page(self):
        # given
        for i in range(70):
            Groups.objects.create(owner=self.user, name=f"Group {i}", is_active=True)
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:groups"), {"page": "999"})
        request.user = self.user
        # when
        response = broswe_groups(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group 60")

    def test_view_group(self):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:view_group", args=[2]))
        request.user = self.user
        # when
        response = view_group(request, 2)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group Approve")

    @patch("squads.views.groups.SquadGroup.objects.filter")
    def test_view_group_filter(self, mock_filter):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:view_group", args=[2]))
        request.user = self.user
        mock_first = MagicMock()
        mock_first.check_user.return_value = (True, ["Test Filter"])

        mock_filter.return_value.first.return_value = mock_first

        # when
        response = view_group(request, 2)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Group Approve")

    @patch("squads.views.groups.messages")
    def test_view_group_not_exist(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:view_group", args=[9]))
        request.user = self.user
        # when
        response = view_group(request, 9)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()



from memberaudit.models import (
    Character,
    CharacterSkill,
    CharacterSkillSetCheck,
    SkillSet,
    SkillSetSkill,
)

from django.contrib.contenttypes.models import ContentType
from eveuniverse.models import EveGroup, EveType

from squads.models.filters import (
    AssetsFilter,
    CharacterAsset,
    ShipFilter,
    SkillSetFilter,
    SquadFilter,
    SquadGroup,
)
from squads.models.groups import Groups


def load_filters():
    AssetsFilter.objects.all().delete()
    SkillSetFilter.objects.all().delete()
    ShipFilter.objects.all().delete()

    asset_filter = AssetsFilter.objects.create(description="Naglfar Asset Filter")
    asset_filter.assets.add(EveType.objects.get(name="Naglfar"))
    asset_filter.save()
    filter1 = SquadFilter.objects.create(
        object_id=asset_filter.id,  # Use the ID of the asset_filter instance
        content_type=ContentType.objects.get_for_model(AssetsFilter),
    )

    ship_filter = ShipFilter.objects.create(description="Dreadnought Ship Filter")
    ship_filter.ship.add(EveGroup.objects.get(id=485))
    ship_filter.save()

    skill_set_filter = SkillSetFilter.objects.create(description="SkillSet Filter")
    skill_set = SkillSet.objects.create(name="Shield Management Skill Set")
    SkillSetSkill.objects.create(
        skill_set=skill_set,
        eve_type=EveType.objects.get(name="Shield Management"),
        required_level=3,
    )
    skill_set_filter.skill_sets.add(skill_set)
    skill_set_filter.save()
    filter2 = SquadFilter.objects.create(
        object_id=skill_set_filter.id,  # Use the ID of the asset_filter instance
        content_type=ContentType.objects.get_for_model(SkillSetFilter),
    )

    empty_filter = SquadFilter.objects.create(
        object_id=999,  # Kein object_id und content_type gesetzt
        content_type=ContentType.objects.get_for_model(SkillSetFilter),
    )

    CharacterAsset.objects.create(
        item_id=19722,
        character=Character.objects.get(eve_character__character_id=1001),
        eve_type=EveType.objects.get(name="Naglfar"),
        quantity=1,
        is_singleton=False,
    )
    CharacterAsset.objects.create(
        item_id=19722,
        character=Character.objects.get(eve_character__character_id=1002),
        eve_type=EveType.objects.get(name="Naglfar"),
        quantity=1,
        is_singleton=False,
    )

    CharacterSkill.objects.create(
        character=Character.objects.get(eve_character__character_id=1001),
        eve_type=EveType.objects.get(name="Shield Management"),
        active_skill_level=5,
        trained_skill_level=5,
        skillpoints_in_skill=256000,
    )

    CharacterSkill.objects.create(
        character=Character.objects.get(eve_character__character_id=1002),
        eve_type=EveType.objects.get(name="Shield Management"),
        active_skill_level=5,
        trained_skill_level=5,
        skillpoints_in_skill=256000,
    )

    CharacterSkillSetCheck.objects.create(
        character=Character.objects.get(eve_character__character_id=1001),
        skill_set=skill_set,
    )

    squad1 = SquadGroup.objects.create(
        group=Groups.objects.get(name="Group No Approve"),
        description="A test squad",
    )
    squad1.filters.add(filter1)
    squad1.filters.add(filter2)
    squad1.filters.add(empty_filter)



import time
from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import RequestFactory, TestCase

from squads.models.groups import Groups
from squads.tests.testdata.load_users import load_users
from squads.view_helpers.core import generate_unique_id
from squads.views import _core


class CoreTests(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        cls.user = User.objects.get(username="groupuser")
        cls.factory = RequestFactory()
        cls.group = Groups.objects.create(owner=cls.user, name="Test Group")

    @patch("squads.models.member.Pending.objects.filter")
    def test_add_info_to_context(self, mock_filter):
        mock_filter.return_value.count.return_value = 5
        request = self.factory.get("/")
        request.user = self.user
        context = _core.add_info_to_context(request, {})
        self.assertEqual(context["pending_count"], 5)

    @patch("squads.models.groups.Groups.objects.visible_to")
    def test_check_permission_true(self, mock_visible_to):
        mock_visible_to.return_value = [self.group]
        request = self.factory.get("/")
        request.user = self.user
        self.assertTrue(_core.check_permission(request, self.group))

    @patch("squads.models.groups.Groups.objects.visible_to")
    def test_check_permission_false(self, mock_visible_to):
        mock_visible_to.return_value = []
        request = self.factory.get("/")
        request.user = self.user
        self.assertFalse(_core.check_permission(request, self.group))


class TestGenerateUniqueId(TestCase):

    def test_length_of_output(self):
        unique_id = generate_unique_id()
        self.assertEqual(len(unique_id), 12)

    def test_uniqueness_over_time(self):
        unique_ids = set()
        for _ in range(5):
            unique_id = generate_unique_id()
            self.assertNotIn(unique_id, unique_ids)
            unique_ids.add(unique_id)
            time.sleep(0.01)  # Sleep to ensure time-based uniqueness



from unittest.mock import patch

from django.test import TestCase

from squads import __title__, __version__


class TestEsiClientProviderInitialization(TestCase):
    @patch("esi.clients.EsiClientProvider")
    def test_esi_client_provider_initialization(self, mock_esi_client_provider):
        # The esi instance is created when the module is imported, so we need to reload it
        # to apply the mocks. This requires the importlib module.
        import squads.providers

        # Assert that EsiClientProvider was called with the correct app_info_text
        mock_esi_client_provider.assert_called_once_with(
            app_info_text=f"{__title__} v{__version__}"
        )



from http import HTTPStatus

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from squads.tests.testdata.load_groups import load_groups
from squads.tests.testdata.load_users import load_users
from squads.views.main import squads_index, squads_membership, squads_pending


class TestViews(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        load_groups()
        cls.factory = RequestFactory()
        cls.user = User.objects.get(username="groupuser")
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user)

    def test_view(self):
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:index"))
        request.user = self.user
        response = squads_index(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_membership(self):
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:membership"))
        request.user = self.user
        response = squads_membership(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_pending(self):
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:pending"))
        request.user = self.user
        response = squads_pending(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)



from http import HTTPStatus
from unittest.mock import patch

from django.contrib.auth.models import User
from django.http import HttpResponseRedirect
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from squads.models.groups import Groups
from squads.models.member import Memberships, Pending
from squads.tests.testdata.load_groups import load_groups
from squads.tests.testdata.load_users import load_users
from squads.views.application import apply_group, cancel_group, leave_group


class TestApplictaion(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        load_groups()
        cls.factory = RequestFactory()
        cls.user = User.objects.get(username="groupuser")
        cls.user2 = User.objects.get(username="groupuser2")
        cls.group = Groups.objects.get(id=1)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user)

    @patch("squads.views.application.messages")
    def test_apply_group_no_valid_post(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:apply_group", args=[self.group.pk]),
            {"novalid": True, "comment": "Test Comment"},
        )
        request.user = self.user
        # when
        response = apply_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Pending.objects.count(), 0)

    @patch("squads.views.application.messages")
    def test_apply_group(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:apply_group", args=[self.group.pk]),
            {"apply_group": True, "comment": "Test Comment"},
        )
        request.user = self.user
        # when
        response = apply_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Pending.objects.count(), 1)
        self.assertEqual(Pending.objects.first().comment, "Test Comment")
        self.assertEqual(Pending.objects.first().group, self.group)

    @patch("squads.views.application.messages")
    @patch("squads.views.application.CommentForm")
    def test_apply_group_with_comment(self, mock_comment_form, mock_messages):
        # given
        mock_comment_form.return_value.is_valid.return_value = False
        mock_comment_form.return_value.cleaned_data.get.return_value = "Test Comment"

        request = self.factory.post(
            reverse("squads:apply_group", args=[self.group.pk]),
            {"apply_group": True, "comment": "Test Comment"},
        )
        request.user = self.user

        # when
        response = apply_group(request, self.group.pk)
        # then
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual(response.status_code, HTTPStatus.FOUND)

        self.assertEqual(Pending.objects.count(), 1)
        pending = Pending.objects.first()
        self.assertEqual(pending.comment, "")
        self.assertEqual(pending.group, self.group)
        self.assertEqual(pending.user, self.user)

        mock_messages.success.assert_called_once()

    @patch("squads.views.application.messages")
    def test_apply_group_join(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:apply_group", args=[self.group.pk]),
            {
                "join_group": True,
            },
        )
        request.user = self.user
        # when
        response = apply_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Pending.objects.count(), 0)
        self.assertEqual(Memberships.objects.count(), 1)
        self.assertEqual(Memberships.objects.first().group, self.group)

    @patch("squads.views.application.messages")
    def test_leave_group(self, mock_messages):
        # given
        Memberships.objects.create(group=self.group, user=self.user, req_filters=False)
        self.client.force_login(self.user)
        request = self.factory.post(reverse("squads:leave_group", args=[self.group.pk]))
        request.user = self.user
        # when
        response = leave_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Memberships.objects.count(), 0)

        mock_messages.warning.assert_called_once()

    @patch("squads.views.application.messages")
    def test_leave_group_not_in_squad(self, mock_messages):
        # given
        Memberships.objects.create(group=self.group, user=self.user2, req_filters=False)
        self.client.force_login(self.user)
        request = self.factory.post(reverse("squads:leave_group", args=[self.group.pk]))
        request.user = self.user
        # when
        response = leave_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Memberships.objects.count(), 1)
        mock_messages.error.assert_called_once()

    @patch("squads.views.application.messages")
    def test_cancel_group(self, mock_messages):
        # given
        Pending.objects.create(group=self.group, user=self.user, req_filters=False)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:cancel_group", args=[self.group.pk])
        )
        request.user = self.user
        # when
        response = cancel_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Pending.objects.count(), 0)
        mock_messages.warning.assert_called_once()

    @patch("squads.views.application.messages")
    def test_cancel_group_not_applied(self, mock_messages):
        # given
        Pending.objects.create(group=self.group, user=self.user2, req_filters=False)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:cancel_group", args=[self.group.pk])
        )
        request.user = self.user
        # when
        response = cancel_group(request, self.group.pk)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(Pending.objects.count(), 1)
        mock_messages.error.assert_called_once()



"""
Memberships Model
"""

from memberaudit.models import (
    CharacterAsset,
    CharacterSkill,
    CharacterSkillSetCheck,
    SkillSet,
    SkillSetSkill,
)

from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _
from eveuniverse.models import EveGroup, EveType

from squads.hooks import get_extension_logger
from squads.models.groups import Groups

logger = get_extension_logger(__name__)


class BaseFilter(models.Model):
    """BaseFilter Model."""

    description = models.CharField(
        max_length=500,
        help_text=_("Filter Name."),
    )

    def __str__(self) -> str:
        return str(self.description)

    class Meta:
        abstract = True


class SkillSetFilter(BaseFilter):
    """SkillSetFilter."""

    skill_sets = models.ManyToManyField(
        SkillSet,
        help_text=_(
            "Users must possess the selected skill sets with at least <strong>one</strong> character."
        ),
        related_name="squads_skill_sets",
    )

    def check_filter(self, user: User):
        """
        Returns:
            bool: True if the user passes the filter, False otherwise.
            list: A list dict of missing skills that the user's Characters do not have.
        """
        missing = []
        required_skills = set(self.skill_sets.all())

        user_skills = CharacterSkillSetCheck.objects.filter(
            character__eve_character__character_ownership__user=user,
            skill_set__in=required_skills,
            failed_required_skills__isnull=True,
        )
        print(user_skills)
        if not user_skills.exists():
            print("No user skills")
            existing_skills = set(user_skills.values_list("skill_set", flat=True))
            missing_skills = required_skills - existing_skills

            for missing_skill in missing_skills:
                skills_with_levels = SkillSetSkill.objects.filter(
                    skill_set=missing_skill
                ).values_list("eve_type", "eve_type__name", "required_level")

                for skill, skill_name, required_level in skills_with_levels:
                    char_skills = CharacterSkill.objects.select_related(
                        "character__eve_character__character_ownership"
                    ).filter(
                        character__eve_character__character_ownership__user=user,
                        eve_type=skill,
                        trained_skill_level__gte=required_level,
                    )

                    if not char_skills.exists():
                        missing.append(
                            {
                                "type": "Skill",
                                "name": skill_name,
                                "amount": f"Lv. {required_level}",
                            }
                        )

        return user_skills.exists(), missing


class AssetsFilter(BaseFilter):
    """AssetsFilter."""

    assets = models.ManyToManyField(
        EveType,
        help_text=_(
            "Users must have at least <strong>one</strong> of the selected assets with <strong>one</strong> character."
        ),
    )

    def check_filter(self, user: User):
        """
        Returns:
            bool: True if the user passes the filter, False otherwise.
            list: A list dict of missing assets that the user's Characters do not have.
        """
        missing = []
        required_assets = set(self.assets.all().values_list("name", flat=True))

        user_assets = CharacterAsset.objects.filter(
            character__eve_character__character_ownership__user=user,
            eve_type__in=self.assets.all(),
        )

        existing_assets = set()
        if user_assets.exists():
            existing_assets = set(user_assets.values_list("eve_type__name", flat=True))

        missing_assets = required_assets - existing_assets

        missing_assets_string = ", ".join(sorted(missing_assets))

        if missing_assets and not user_assets.exists():
            missing.append(
                {
                    "type": "Asset" if len(missing_assets) == 1 else "Asset one of",
                    "name": missing_assets_string,
                    "amount": "",
                }
            )

        return user_assets.exists(), missing


class ShipFilter(BaseFilter):
    """ShipFilter."""

    ship = models.ManyToManyField(
        EveGroup,
        help_text=_(
            "Users must have at least <strong>one</strong> of the selected Ship Type with <strong>one</strong> character."
        ),
    )

    def check_filter(self, user: User):
        """
        Returns:
            bool: True if the user passes the filter, False otherwise.
            list: A list dict of missing Ship Type that the user's Characters do not have.
        """
        missing = []
        required_ship = set(self.ship.all().values_list("name", flat=True))

        user_ship = CharacterAsset.objects.filter(
            character__eve_character__character_ownership__user=user,
            eve_type__eve_group_id__in=self.ship.all(),
        )

        existing_ship = set()
        if user_ship.exists():
            existing_ship = set(
                user_ship.values_list("eve_type__eve_group_id__name", flat=True)
            )

        missing_ship = required_ship - existing_ship

        missing_ship_string = ", ".join(sorted(missing_ship))

        if missing_ship and not user_ship.exists():
            missing.append(
                {"type": "Ship Type", "name": missing_ship_string, "amount": ""}
            )

        return user_ship.exists(), missing


class SquadFilter(models.Model):
    """SquadFilter show up all Filters."""

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, editable=False
    )
    object_id = models.PositiveIntegerField(editable=False)
    filter_object = GenericForeignKey("content_type", "object_id")

    def __str__(self) -> str:
        return str(self.filter_object)

    class Meta:
        verbose_name = "Squad Filter"


class SquadGroup(models.Model):
    """Squads Group Filter."""

    group = models.OneToOneField(Groups, on_delete=models.CASCADE)
    description = models.CharField(max_length=500, default="", blank=True)
    filters = models.ManyToManyField(SquadFilter)
    enabled = models.BooleanField(default=True)  # Not Implemented yet
    last_updated = models.DateTimeField(auto_now=True)

    def __str__(self) -> str:
        return str(f"Filter Group: {self.group.name}")

    # Check Filters
    # pylint: disable=broad-exception-caught
    def run_filters(self, user: User):
        output = []
        missing_req = []
        for check in self.filters.all():
            try:
                _filter = check.filter_object
                if _filter is None:
                    logger.debug("Filter %s is None", check)
                    continue
                run_test, missing = _filter.check_filter(user)
                if missing:
                    missing_req.append(missing)
            except Exception as e:
                run_test = False
                logger.warning("Filter Check Failed: %s", check)
                logger.debug("Error: %s", e, exc_info=True)
            _check = {
                "desc": check.filter_object.description,
            }
            _check["result"] = run_test
            _check["filter"] = check
            output.append(_check)
        return output, missing_req

    def process_filters(self, checks):
        output = True
        for check in checks:
            output = output and check.get("result", False)
        return output

    def check_user(self, user: User):
        checks, missing = self.run_filters(user)
        output = self.process_filters(checks)
        logger.debug("Output: %s and Missing: %s", output, missing)
        return output, missing



from http import HTTPStatus
from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from squads.models.groups import Groups
from squads.tests.testdata.load_groups import load_groups
from squads.tests.testdata.load_users import load_users
from squads.views.manage import delete_group, edit_group, manage_groups


class TestGroupManagement(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        load_groups()
        cls.factory = RequestFactory()
        cls.user = User.objects.get(username="groupuser")
        cls.user2 = User.objects.get(username="groupuser2")
        cls.group = Groups.objects.get(id=1)
        cls.group2 = Groups.objects.get(id=2)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", cls.user)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user2)
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", cls.user2)

    def test_manage_groups(self):
        # given
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        # when
        request = self.factory.get(reverse("squads:manage_groups"))
        request.user = self.user
        response = manage_groups(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    @patch("squads.views.manage.messages")
    def test_delete_group(self, mock_messages):
        # given
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(reverse("squads:delete_group", args=[1]))
        request.user = self.user
        # when
        response = delete_group(request, 1)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertFalse(Groups.objects.filter(id=1).exists())
        mock_messages.success.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_delete_group_not_exist(self, mock_messages):
        # given
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(reverse("squads:delete_group", args=[9]))
        request.user = self.user
        # when
        response = delete_group(request, 9)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_delete_group_no_permission(self, mock_messages):
        # given
        self.client.force_login(self.user2)
        request = self.factory.post(reverse("squads:delete_group", args=[3]))
        request.user = self.user2
        # when
        response = delete_group(request, 3)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)

        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_edit_group(self, mock_messages):
        # given
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:edit_group", args=[1]),
            {
                "name": "Test Group",
                "description": "Test Description",
                "req_approve": True,
            },
        )
        request.user = self.user
        # when
        response = edit_group(request, 1)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertTrue(
            Groups.objects.filter(
                id=1,
                name="Test Group",
                description="Test Description",
                req_approve=True,
            ).exists()
        )
        mock_messages.success.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_edit_group_not_exist(self, mock_messages):
        # given
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:edit_group", args=[9]),
            {
                "name": "Test Group",
                "description": "Test Description",
                "req_approve": True,
            },
        )
        request.user = self.user
        # when
        response = edit_group(request, 9)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_edit_group_no_permission(self, mock_messages):
        # given
        self.client.force_login(self.user2)
        request = self.factory.post(
            reverse("squads:edit_group", args=[3]),
            {
                "name": "Test Group",
                "description": "Test Description",
                "req_approve": True,
            },
        )
        request.user = self.user2
        # when
        response = edit_group(request, 3)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_edit_group_invalid_form(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:edit_group", args=[1]),
            {"name": "", "description": "Test Description"},
        )
        request.user = self.user
        # when
        response = edit_group(request, 1)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_edit_group_no_post(self, mock_messages):
        # given
        self.client.force_login(self.user)
        request = self.factory.get(reverse("squads:edit_group", args=[1]))
        request.user = self.user
        # when
        response = edit_group(request, 1)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    @patch("squads.views.manage.messages")
    def test_edit_group_with_image(self, mock_messages):
        # given
        self.client.force_login(self.user)
        group = Groups.objects.get(id=1)
        group.image = "existing/image.png"
        group.save()
        request = self.factory.post(
            reverse("squads:edit_group", args=[1]),
            {"name": "Test Group", "description": "Test Description"},
        )
        request.user = self.user
        # when
        response = edit_group(request, 1)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.success.assert_called_once()



from unittest.mock import MagicMock, patch

from django.test import TestCase

from squads.models.filters import AssetsFilter, SkillSetFilter
from squads.signals import HookCache, new_filter, rem_filter


class TestHookCache(TestCase):
    @patch("squads.signals.hooks.get_hooks")
    def test_get_hooks(self, mock_get_hooks):
        # given/when
        mock_get_hooks.return_value = [lambda: [SkillSetFilter], lambda: [AssetsFilter]]
        cache = HookCache()
        hooks = cache.get_hooks()
        self.assertEqual(len(hooks), 2)
        self.assertTrue(SkillSetFilter in hooks)
        self.assertTrue(AssetsFilter in hooks)
        # then
        hooks_again = cache.get_hooks()
        self.assertEqual(hooks, hooks_again)
        mock_get_hooks.assert_called_once()

    @patch("squads.signals.hooks.get_hooks")
    def test_ignores_duplicate_hooks(self, mock_get_hooks):
        # given/when
        mock_get_hooks.return_value = [
            lambda: [SkillSetFilter],
            lambda: [SkillSetFilter],
        ]
        cache = HookCache()
        hooks = cache.get_hooks()
        # then
        self.assertEqual(len(hooks), 1)
        self.assertTrue(SkillSetFilter in hooks)


class TestNewFilterSignal(TestCase):
    @patch("squads.signals.model.SquadFilter.objects.create")
    def test_new_filter_created(self, mock_create):
        mock_instance = MagicMock()
        new_filter(sender=None, instance=mock_instance, created=True)
        mock_create.assert_called_once_with(filter_object=mock_instance)

    @patch("squads.signals.model.SquadFilter.objects.create")
    def test_new_filter_not_created(self, mock_create):
        mock_instance = MagicMock()
        new_filter(sender=None, instance=mock_instance, created=False)
        mock_create.assert_not_called()

    @patch("squads.signals.logger.error")
    def test_exception_logging(self, mock_logger):
        new_filter(sender=None, instance=None, created=True)
        mock_logger.assert_called_once()


class TestRemoveFilterSignal(TestCase):
    @patch("squads.signals.model.SquadFilter.objects.get")
    def test_remove_filter(self, mock_get):
        mock_instance = MagicMock()
        mock_instance.pk = 1
        mock_instance.__class__.__name__ = "TestModel"
        mock_filter = MagicMock()
        mock_get.return_value = mock_filter
        rem_filter(sender=None, instance=mock_instance)
        mock_get.assert_called_once()
        mock_filter.delete.assert_called_once()

    @patch("squads.signals.logger.error")
    @patch("squads.signals.model.SquadFilter.objects.get")
    def test_exception_logging(self, mock_get, mock_logger):
        mock_filter = MagicMock()
        mock_get.return_value = mock_filter
        rem_filter(sender=None, instance=None)
        mock_logger.assert_called_once()



"""Generate AllianceAuth test objects from allianceauth.json."""

import json
from pathlib import Path

from django.contrib.auth.models import User

from squads.models.groups import Groups


def load_users():
    User.objects.all().delete()

    User.objects.create_user(username="groupuser", password="testpassword1")
    User.objects.create_user(username="groupuser2", password="testpassword2")
    User.objects.create_user(
        username="groupuser3", password="testpassword3", is_staff=True
    )
    User.objects.create_user(
        username="groupuser4", password="testpassword4", is_superuser=True
    )



# Import necessary modules and classes
from memberaudit.models import Character

from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.test import TestCase
from eveuniverse.models import EveGroup, EveType

from squads.admin import (
    AssetsFilterAdmin,
    ShipFilterForm,
    ShippFilterAdmin,
    SkillSetFilterAdmin,
    SquadFilterAdmin,
)
from squads.models.filters import (
    AssetsFilter,
    CharacterAsset,
    ShipFilter,
    SkillSet,
    SkillSetFilter,
    SquadFilter,
)
from squads.tests.testdata.load_allianceauth import load_allianceauth
from squads.tests.testdata.load_eveuniverse import load_eveuniverse
from squads.tests.testdata.load_memberaudit import load_memberaudit
from squads.tests.testdata.load_users import load_users


class SkillSetFilterAdminTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Setup test environment
        cls.site = AdminSite()
        load_users()
        cls.user = User.objects.get(username="groupuser4")

        # Create test data
        cls.skill_set_filter = SkillSetFilter.objects.create(description="Test Filter")
        cls.skill_set = SkillSet.objects.create(name="Test Skill Set")
        cls.skill_set_filter.skill_sets.add(cls.skill_set)

    def test_skill_sets_display(self):
        self.client.force_login(self.user)

        admin = SkillSetFilterAdmin(SkillSetFilter, self.site)

        result = admin._skill_sets(self.skill_set_filter)

        # Assert the expected result
        self.assertEqual(result, "Test Skill Set")

    def test_skill_queryset(self):
        self.client.force_login(self.user)

        admin = SkillSetFilterAdmin(SkillSetFilter, self.site)

        # Call the get_queryset method
        result = admin.get_queryset(self.site)

        # Assert the expected result
        self.assertIsNotNone(result)


class AssetFilterAdminTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.site = AdminSite()
        load_users()
        load_allianceauth()
        load_eveuniverse()
        load_memberaudit()
        cls.user = User.objects.get(username="groupuser4")

        # Create test data
        cls.asset_filter = AssetsFilter.objects.create(description="Test Filter")
        cls.character = Character.objects.first()
        cls.asset = CharacterAsset.objects.create(
            item_id=19722,
            character=cls.character,
            eve_type=EveType.objects.get(name="Naglfar"),
            quantity=1,
            is_singleton=False,
        )
        cls.asset_filter.assets.add(EveType.objects.get(name="Naglfar"))

    def test_assets_display(self):
        self.client.force_login(self.user)

        admin = AssetsFilterAdmin(AssetsFilter, self.site)

        result = admin._assets(self.asset_filter)

        # Assert the expected result
        self.assertEqual(result, "Naglfar")

    def test_asset_queryset(self):
        self.client.force_login(self.user)

        admin = AssetsFilterAdmin(AssetsFilter, self.site)

        # Call the get_queryset method
        result = admin.get_queryset(self.site)

        # Assert the expected result
        self.assertIsNotNone(result)


class ShipFilterAdminTest(TestCase):
    def setUp(self):
        # Setup test environment
        self.site = AdminSite()
        load_users()
        load_allianceauth()
        load_eveuniverse()
        load_memberaudit()
        self.user = User.objects.get(username="groupuser4")

        # Create test data
        self.ship_filter = ShipFilter.objects.create(description="Test Filter")
        self.character = Character.objects.first()
        self.asset = CharacterAsset.objects.create(
            item_id=19722,
            character=self.character,
            eve_type=EveType.objects.get(name="Naglfar"),
            quantity=1,
            is_singleton=False,
        )
        self.ship_filter.ship.add(EveGroup.objects.get(id=485))

    def test_ship_display(self):
        self.client.force_login(self.user)

        admin = ShippFilterAdmin(ShipFilter, self.site)

        result = admin._ship(self.ship_filter)

        # Assert the expected result
        self.assertEqual(result, "Dreadnought")

    def test_ship_queryset(self):
        self.client.force_login(self.user)

        admin = ShippFilterAdmin(ShipFilter, self.site)

        # Call the get_queryset method
        result = admin.get_queryset(self.site)

        # Assert the expected result
        self.assertIsNotNone(result)

    def test_ship_field_queryset(self):
        form = ShipFilterForm()
        ship_field_qs = form.fields["ship"].queryset

        # Verify queryset
        self.assertTrue(ship_field_qs.filter(name="Dreadnought").exists())
        self.assertFalse(ship_field_qs.filter(name="Structure").exists())
        self.assertFalse(ship_field_qs.filter(name="Planet").exists())


class SquadFilterAdminTest(TestCase):
    def setUp(self):
        load_users()
        self.site = AdminSite()
        self.user = User.objects.get(username="groupuser4")

    def test_has_add_permission(self):
        # Create an instance of the admin class
        admin = SquadFilterAdmin(SquadFilter, self.site)

        # Create a mock request object
        request = self.user

        # Call the has_add_permission method and assert the result is False
        self.assertFalse(admin.has_add_permission(request))



from unittest import TestCase
from unittest.mock import MagicMock

from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db import models
from django.db.models.fields.files import ImageField, ImageFieldFile

from app_utils.testing import create_user_from_evecharacter

from squads.forms import CustomClearableFileInput, SquadsGroupForm
from squads.models import Groups
from squads.tests.testdata.load_allianceauth import load_allianceauth


class TestCleanImage(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )
        cls.form = SquadsGroupForm()
        cls.group = Groups(
            owner=cls.user,
            name="Test Group",
            description="Test Description",
            req_approve=True,
        )
        cls.group.save()
        cls.form = SquadsGroupForm(instance=cls.group)

    def test_init_with_instance_pk(self):
        self.assertIn("is_active", self.form.fields)
        self.assertFalse(self.form.fields["is_active"].required)
        self.assertEqual(self.form.fields["is_active"].initial, self.group.is_active)
        self.assertEqual(
            self.form.fields["is_active"].widget.attrs, {"class": "form-check-input"}
        )

    def test_form_image_valid_size(self):
        image_mock = MagicMock()
        image_mock.size = 1 * 1024 * 1024

        self.form.cleaned_data = {"image": image_mock}

        result = self.form.clean_image()
        self.assertEqual(result, image_mock)

    def test_form_image_too_large(self):
        image_mock = MagicMock()
        image_mock.size = 3 * 1024 * 1024

        self.form.cleaned_data = {"image": image_mock}

        with self.assertRaises(ValidationError):
            self.form.clean_image()

    def test_form_no_image(self):
        self.form.cleaned_data = {"image": None}

        result = self.form.clean_image()
        self.assertIsNone(result)

    def test_form_image_not_found(self):
        image_content = b"\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x64\x00\x00\x00\x64\x08\x06\x00\x00\x00\x70\xe2\x95n\x00\x00\x00\x0bIDATx\x9cc```\xf8\xff\xff?\x03\x00\x18\x0d\x04\x04Q\xd4\xe8v\x00\x00\x00\x00IEND\xaeB`\x82"
        uploaded_image = SimpleUploadedFile(
            name="test_image.png", content=image_content, content_type="image/png"
        )

        image_field = ImageField()

        image_field_file = ImageFieldFile(
            instance=None, field=image_field, name=uploaded_image.name
        )
        image_field_file.file = uploaded_image

        self.form.cleaned_data = {"image": image_field_file}
        result = self.form.clean_image()
        self.assertIsNone(result)

    def test_form_text_to_long(self):
        self.form.cleaned_data = {"name": "a" * 27}
        with self.assertRaises(ValidationError):
            self.form.clean_name()


class CustomClearableFileInputTest(TestCase):
    def test_is_initial_set_to_false_if_value_has_url(self):
        widget = CustomClearableFileInput()
        value = MagicMock(url="http://example.com/image.png")
        context = widget.get_context("image", value, {})
        self.assertFalse(context["widget"]["is_initial"])

    def test_is_initial_true_or_not_set_if_value_has_no_url(self):
        widget = CustomClearableFileInput()
        value = MagicMock(spec=[])
        context = widget.get_context("image", value, {})
        is_initial = context["widget"].get("is_initial", False)
        self.assertFalse(is_initial)



# Generated by Django 4.2.11 on 2024-06-11 20:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("squads", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="groups",
            old_name="require_approval",
            new_name="req_approve",
        ),
        migrations.RenameField(
            model_name="memberships",
            old_name="has_required_skills",
            new_name="req_filters",
        ),
        migrations.RenameField(
            model_name="pending",
            old_name="application",
            new_name="comment",
        ),
        migrations.AddField(
            model_name="pending",
            name="req_filters",
            field=models.BooleanField(default=True),
        ),
    ]



from unittest.mock import MagicMock, patch

from django.contrib.auth.models import User
from django.test import TestCase

from squads.models.groups import Groups
from squads.models.member import Memberships, Pending
from squads.tasks import run_check_members, run_check_pendings, run_check_squads
from squads.tests.testdata.load_groups import load_groups, load_membership, load_pending
from squads.tests.testdata.load_users import load_users


class TestTasks(TestCase):
    @classmethod
    def setUpClass(self):
        super().setUpClass()
        load_users()
        load_groups()
        load_membership()
        load_pending()

    @patch("squads.tasks.logger")
    def test_run_check_squads(self, mock_logger):
        # given/when
        run_check_squads()
        # then
        mock_logger.info.assert_called_with("Check Squads runs completed: %s", 2)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_members(self, mock_squadgroup_filter):
        # given
        mock_filter = MagicMock()
        mock_filter.check_user.return_value = (True, None)
        mock_squadgroup_filter.return_value.first.return_value = mock_filter
        user = User.objects.get(username="groupuser")
        member_actual = Memberships.objects.get(user=user)
        # when
        result = run_check_members(1)
        # then
        member_then = Memberships.objects.get(user=user)
        self.assertFalse(member_actual.req_filters)
        self.assertTrue(member_then.req_filters)
        self.assertTrue(result)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_members_fail_check(self, mock_squadgroup_filter):
        # given
        mock_filter = MagicMock()
        mock_filter.check_user.return_value = (False, None)
        mock_squadgroup_filter.return_value.first.return_value = mock_filter
        user = User.objects.get(username="groupuser2")
        member_actual = Memberships.objects.get(user=user)
        # when
        result = run_check_members(2)
        # then
        member_then = Memberships.objects.get(user=user)
        self.assertTrue(member_actual.req_filters)
        self.assertFalse(member_then.req_filters)
        self.assertTrue(result)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_members_no_filters(self, mock_squadgroup_filter):
        # given
        mock_squadgroup_filter.return_value.first.return_value = False
        # when
        result = run_check_members(1)
        # then
        self.assertFalse(result)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_pending(self, mock_squadgroup_filter):
        # given
        mock_filter = MagicMock()
        mock_filter.check_user.return_value = (True, None)
        mock_squadgroup_filter.return_value.first.return_value = mock_filter
        user = User.objects.get(username="groupuser")
        pending_actual = Pending.objects.get(user=user)
        # when
        result = run_check_pendings(1)
        # then
        pending_then = Pending.objects.get(user=user)
        self.assertFalse(pending_actual.req_filters)
        self.assertTrue(pending_then.req_filters)
        self.assertTrue(result)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_pending_fail_check(self, mock_squadgroup_filter):
        # given
        mock_filter = MagicMock()
        mock_filter.check_user.return_value = (False, None)
        mock_squadgroup_filter.return_value.first.return_value = mock_filter
        user = User.objects.get(username="groupuser2")
        pending_actual = Pending.objects.get(user=user)
        # when
        result = run_check_pendings(2)
        # then
        pending_then = Pending.objects.get(user=user)
        self.assertTrue(pending_actual.req_filters)
        self.assertFalse(pending_then.req_filters)
        self.assertTrue(result)

    @patch("squads.tasks.SquadGroup.objects.filter")
    def test_run_check_pending_no_filters(self, mock_squadgroup_filter):
        # given
        mock_squadgroup_filter.return_value.first.return_value = False
        # when
        result = run_check_pendings(1)
        # then
        self.assertFalse(result)



from http import HTTPStatus
from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from squads.models.groups import Groups
from squads.models.member import Memberships, Pending
from squads.tests.testdata.load_groups import load_groups, load_pending
from squads.tests.testdata.load_users import load_users
from squads.views.manage import (
    manage_application_accept,
    manage_application_decline,
    manage_pendings,
)


class TestManageApplicationManagement(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_users()
        load_groups()
        cls.factory = RequestFactory()
        cls.user = User.objects.get(username="groupuser")
        cls.user2 = User.objects.get(username="groupuser2")
        cls.group = Groups.objects.get(id=1)
        cls.group2 = Groups.objects.get(id=2)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", cls.user)
        AuthUtils.add_permission_to_user_by_name("squads.basic_access", cls.user2)
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", cls.user2)

    @patch("squads.views.manage.messages")
    def test_manage_application_accept(self, mock_messages):
        # given
        load_pending()
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", self.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:accept_group", args=["cf2605fa1ff2"])
        )
        request.user = self.user
        # when
        response = manage_application_accept(request, "cf2605fa1ff2")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertTrue(
            Memberships.objects.filter(user=self.user2, group=self.group2).exists()
        )
        self.assertFalse(
            Pending.objects.filter(
                user=self.user2, group=self.group2, application_id="cf2605fa1ff2"
            ).exists()
        )
        mock_messages.success.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_manage_application_accept_not_exist(self, mock_messages):
        # given
        load_pending()
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", self.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:accept_group", args=["cf2605fa1ff9"])
        )
        request.user = self.user
        # when
        response = manage_application_accept(request, "cf2605fa1ff9")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_manage_application_accept_no_permission(self, mock_messages):
        # given
        load_pending()
        self.client.force_login(self.user2)
        request = self.factory.post(
            reverse("squads:accept_group", args=["cf2605fa1ff3"])
        )
        request.user = self.user2
        # when
        response = manage_application_accept(request, "cf2605fa1ff3")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_manage_application_decline(self, mock_messages):
        # given
        load_pending()
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", self.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:decline_group", args=["cf2605fa1ff2"])
        )
        request.user = self.user
        # when
        response = manage_application_decline(request, "cf2605fa1ff2")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertFalse(
            Pending.objects.filter(
                user=self.user2, group=self.group2, application_id="cf2605fa1ff2"
            ).exists()
        )
        mock_messages.success.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_manage_application_decline_not_exist(self, mock_messages):
        # given
        load_pending()
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", self.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        request = self.factory.post(
            reverse("squads:decline_group", args=["cf2605fa1ff9"])
        )
        request.user = self.user
        # when
        response = manage_application_decline(request, "cf2605fa1ff9")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    @patch("squads.views.manage.messages")
    def test_manage_application_decline_no_permission(self, mock_messages):
        # given
        load_pending()
        self.client.force_login(self.user2)
        request = self.factory.post(
            reverse("squads:decline_group", args=["cf2605fa1ff3"])
        )
        request.user = self.user2
        # when
        response = manage_application_decline(request, "cf2605fa1ff3")
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once()

    def test_manage_pendings(self):
        # given
        load_pending()
        AuthUtils.add_permission_to_user_by_name("squads.squad_manager", self.user)
        AuthUtils.add_permission_to_user_by_name("squads.squad_admin", self.user)
        self.client.force_login(self.user)
        # when
        request = self.factory.get(reverse("squads:manage_pendings"))
        request.user = self.user
        response = manage_pendings(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)



from django.contrib.auth.models import Permission
from django.test import TestCase

from app_utils.testing import create_user_from_evecharacter

from squads.models.groups import Groups
from squads.tests.testdata.load_allianceauth import load_allianceauth


class GroupsManagerTest(TestCase):
    def setUp(self):
        load_allianceauth()
        self.groups = Groups.objects
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )
        self.user2, self.character_ownership = create_user_from_evecharacter(
            1002,
            permissions=[
                "squads.basic_access",
            ],
        )
        Groups.objects.create(
            name="Test Squad",
            description="Test Squad Description",
            owner=self.user2,
        )
        Groups.objects.create(
            name="Test Squad 2",
            description="Test Squad Description",
            owner=self.user,
        )

    def test_visible_to_superuser(self):
        # given
        self.user.is_superuser = True
        # when
        visible_squads = self.groups.visible_to(self.user)
        # then
        self.assertEqual(visible_squads.count(), Groups.objects.count())

    def test_visible_to_squad_admin(self):
        # given
        permission, _ = Permission.objects.get_or_create(codename="squad_admin")
        self.user.user_permissions.add(permission)
        # when
        visible_squads = self.groups.visible_to(self.user)
        # then
        self.assertEqual(visible_squads.count(), Groups.objects.count())

    def test_visible_to_squad_manager(self):
        # given
        permission, _ = Permission.objects.get_or_create(codename="squad_manager")
        self.user.user_permissions.add(permission)
        # when
        visible_squads = self.groups.visible_to(self.user)
        # then
        self.assertEqual(visible_squads.count(), 1)  # Only owns 1 squad

    def test_visible_to_regular_user(self):
        # given/when
        visible_squads = self.groups.visible_to(self.user)
        # then
        self.assertEqual(visible_squads.count(), 0)  # Should see none



from django.test import TestCase

from squads.errors import CustomError, DatabaseError, MemberNotActive, TokenDoesNotExist


class TestCustomExceptions(TestCase):
    def test_token_does_not_exist(self):
        with self.assertRaises(TokenDoesNotExist):
            raise TokenDoesNotExist("Sample message for TokenDoesNotExist")

    def test_member_not_active(self):
        with self.assertRaises(MemberNotActive):
            raise MemberNotActive("Sample message for MemberNotActive")

    def test_database_error(self):
        with self.assertRaises(DatabaseError):
            raise DatabaseError("Sample message for DatabaseError")

    def test_custom_error(self):
        with self.assertRaises(CustomError):
            raise CustomError("Sample message for CustomError")



"""App Tasks"""

from celery import chain as Chain
from celery import shared_task

from squads.hooks import get_extension_logger
from squads.models import Groups, Memberships, Pending
from squads.models.filters import SquadGroup

logger = get_extension_logger(__name__)


@shared_task
def run_check_squads(runs: int = 0):
    """Run Check Squads Filters."""
    groups = Groups.objects.filter(is_active=True).all()

    for group in groups:
        logger.debug("Checking group: %s", group.name)
        runs = runs + 1
        Chain(run_check_members.si(group.pk)).delay()
        Chain(run_check_pendings.si(group.pk)).delay()

    logger.info("Check Squads runs completed: %s", runs)


# TODO Maybe delete Member on filter fail??
@shared_task
def run_check_members(group_id: int, runs: int = 0):
    """Run Check Members."""
    group = Groups.objects.get(id=group_id)
    filters = SquadGroup.objects.filter(group=group).first()
    changed_state = 0

    if filters:
        members = Memberships.objects.filter(group=group).all()
        for member in members:
            filter_req, _ = filters.check_user(member.user)
            runs = runs + 1
            if filter_req:
                member.req_filters = True
                member.save()
            else:
                member.req_filters = False
                member.save()
                changed_state += 1
        logger.info(
            "Member Check %s runs completed: %s, Changed: %s",
            group.name,
            runs,
            changed_state,
        )
        return True
    return False


# TODO Maybe delete Member on filter fail??
@shared_task
def run_check_pendings(group_id: int, runs: int = 0):
    """Run Check Pendings."""
    group = Groups.objects.get(id=group_id)
    filters = SquadGroup.objects.filter(group=group).first()
    changed_state = 0

    if filters:
        pendings = Pending.objects.filter(group=group).all()
        for pending in pendings:
            filter_req, _ = filters.check_user(pending.user)
            runs = runs + 1
            if filter_req:
                pending.req_filters = True
                pending.save()
            else:
                pending.req_filters = False
                pending.save()
                changed_state += 1
        logger.info(
            "Pending Check %s runs completed: %s, Changed: %s",
            group.name,
            runs,
            changed_state,
        )
        return True
    return False



"""Forms for the Squads app."""

import bleach
from bleach.css_sanitizer import CSSSanitizer

from django import forms
from django.core.exceptions import ValidationError
from django.forms.widgets import ClearableFileInput
from django.utils.translation import gettext_lazy as _

from squads.hooks import get_extension_logger

from .models import Groups

logger = get_extension_logger(__name__)


class CustomClearableFileInput(ClearableFileInput):
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        if value and hasattr(value, "url"):
            context["widget"]["is_initial"] = False
        return context


class SquadsGroupForm(forms.ModelForm):
    class Meta:
        model = Groups
        fields = ["name", "description", "req_approve", "is_active", "image"]
        widgets = {
            "image": CustomClearableFileInput(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["name"].widget.attrs.update(
            {"class": "form-control", "maxlength": "26", "required": True}
        )
        self.fields["description"].widget.attrs.update(
            {"class": "form-control", "maxlength": "1000", "rows": "5"}
        )
        self.fields["req_approve"].widget.attrs.update({"class": "form-check-input"})
        self.fields["is_active"].widget.attrs.update({"class": "form-check-input"})

    def clean_name(self):
        name = self.cleaned_data.get("name")
        if name and len(name) > 26:
            raise ValidationError("The name must be at most 26 characters long.")
        return name

    # Prvent Security Issues
    def clean_description(self):
        allowed_tags = [
            "p",
            "b",
            "i",
            "u",
            "a",
            "ul",
            "ol",
            "li",
            "br",
            "img",
            "strong",
            "em",
            "iframe",
            "div",
            "blockquote",
            "span",
        ]
        allowed_attrs = {
            "a": ["href", "title", "style"],
            "img": ["src", "alt", "style"],
            "iframe": ["src", "allowfullscreen", "width", "height", "style"],
            "p": ["style"],
            "div": ["style"],
            "span": ["style"],
        }

        allowed_styles = [
            "color",
            "font-family",
            "font-size",
            "font-style",
            "font-weight",
            "text-align",
            "text-decoration",
            "text-indent",
            "background-color",
            "background-image",
            "background-repeat",
            "background-size",
            "border",
            "border-bottom",
            "border-left",
            "border-radius",
            "border-right",
            "border-top",
            "margin",
            "margin-bottom",
            "margin-left",
            "margin-right",
            "margin-top",
            "padding",
            "padding-bottom",
            "padding-left",
            "padding-right",
            "padding-top",
            "line-height",
            "letter-spacing",
            "word-spacing",
            "width",
            "height",
            "max-width",
            "max-height",
            "min-width",
        ]

        css_santizer = CSSSanitizer(allowed_css_properties=allowed_styles)

        cleaned_data = bleach.clean(
            self.cleaned_data["description"],
            tags=allowed_tags,
            attributes=allowed_attrs,
            strip=True,
            css_sanitizer=css_santizer,
        )

        if len(cleaned_data) > 1000:
            raise ValidationError(
                "The Description is too long. Please keep it under 1000 characters."
            )

        return cleaned_data

    def clean_image(self):
        try:
            image = self.cleaned_data.get("image")
            if image:
                max_size = 2 * 1024 * 1024  # 2 MB
                if image.size > max_size:
                    raise ValidationError(
                        _(
                            "The image file is too large ( > 2 MB ). Please choose a smaller file."
                        )
                    )
        # TODO Fix this in correct way
        # Bad Fix but works for now
        except FileNotFoundError:
            return None
        return image


class CommentForm(forms.Form):
    comment = forms.CharField(
        widget=forms.TextInput(
            attrs={
                "class": "form-control",
                "rows": 1,
                "maxlength": 200,
                "placeholder": "comment...",
            }
        ),
        required=False,
    )



"""Generate AllianceAuth test objects from allianceauth.json."""

import json
import random
from pathlib import Path

from django.contrib.auth.models import User

from squads.models.groups import Groups
from squads.models.member import Memberships, Pending


def load_groups():
    Groups.objects.all().delete()

    Groups.objects.update_or_create(
        id=1,
        name="Group No Approve",
        owner=User.objects.get(username="groupuser"),
        is_active=False,
    )
    Groups.objects.update_or_create(
        id=2,
        name="Group Approve",
        owner=User.objects.get(username="groupuser2"),
        req_approve=True,
    )
    Groups.objects.update_or_create(
        id=3,
        name="Group Staff",
        owner=User.objects.get(username="groupuser3"),
        is_active=False,
        req_approve=True,
    )
    Groups.objects.update_or_create(
        id=4,
        name="Group Superuser",
        owner=User.objects.get(username="groupuser4"),
        is_active=True,
        req_approve=True,
    )


def load_membership():
    Memberships.objects.all().delete()

    group = Groups.objects.get(name="Group No Approve")
    user = User.objects.get(username="groupuser")
    Memberships.objects.create(
        group=group, user=user, req_filters=False, application_id="cf2605fa1ff1"
    )

    group = Groups.objects.get(name="Group Approve")
    user = User.objects.get(username="groupuser2")
    Memberships.objects.create(
        group=group, user=user, req_filters=True, application_id="cf2605fa1ff2"
    )

    group = Groups.objects.get(name="Group Staff")
    user = User.objects.get(username="groupuser3")
    Memberships.objects.create(
        group=group, user=user, req_filters=False, application_id="cf2605fa1ff3"
    )

    group = Groups.objects.get(name="Group Superuser")
    user = User.objects.get(username="groupuser4")
    Memberships.objects.create(
        group=group, user=user, req_filters=True, application_id="cf2605fa1ff4"
    )


def load_pending():
    Pending.objects.all().delete()

    group = Groups.objects.get(name="Group No Approve")
    user = User.objects.get(username="groupuser")
    Pending.objects.create(
        group=group,
        user=user,
        req_filters=False,
        comment="Test Comment",
        application_id="cf2605fa1ff1",
    )

    group = Groups.objects.get(name="Group Approve")
    user = User.objects.get(username="groupuser2")
    Pending.objects.create(
        group=group,
        user=user,
        req_filters=True,
        comment="Test Comment",
        application_id="cf2605fa1ff2",
    )

    group = Groups.objects.get(name="Group Staff")
    user = User.objects.get(username="groupuser3")
    Pending.objects.create(
        group=group, user=user, req_filters=False, application_id="cf2605fa1ff3"
    )

    group = Groups.objects.get(name="Group Superuser")
    user = User.objects.get(username="groupuser4")
    Pending.objects.create(
        group=group, user=user, req_filters=True, application_id="cf2605fa1ff4"
    )



"""
Groups Model
"""

from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import User

from squads.hooks import get_extension_logger
from squads.managers import GroupsManager

logger = get_extension_logger(__name__)


class Groups(models.Model):
    """Groups Model store Groups Data."""

    name = models.CharField(max_length=255, unique=True)
    owner = models.ForeignKey(
        User, on_delete=models.CASCADE, related_name="group_owner"
    )
    description = models.TextField(blank=True)
    is_active = models.BooleanField(default=True)
    req_approve = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    image = models.ImageField(upload_to="squads/groups_images/", blank=True, null=True)

    objects = GroupsManager()

    def __str__(self) -> str:
        return str(self.name)

    class Meta:
        default_permissions = ()
